/**********************************************************
***********************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  imageProcessing.h
* Description:  
*
*
* History:      
* 14-Jun-04     (csabet) Created
***********************************************************
**********************************************************/
#ifndef _IMAGE_PROCESSING_H
#define _IMAGE_PROCESSING_H

/**********************************************************
*   Include files
**********************************************************/

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Enumeration definitions
**********************************************************/

/**********************************************************
*   Type definitions 
**********************************************************/

/**********************************************************
*   Function Prototypes
**********************************************************/
float midiGaussianSmooth (
	float	*array,
	int		length,
	int		peakIn,
	int		searchSpan,
    int		*error);

void normalizeSignal (
	int		arraySize,
	float	*signal);

void computeFrameFlux (
	short int	*inData,
	int			frame,
	int			scalingOffset,
    ImageFormat *format,
	MidiCoords	*target,
	float		*flux,
	int			*pixelCount,
    int			*error);

void computeImageFlux (
	float		*image,
    ImageFormat *format,
	MidiCoords	*target,
	float		*flux,
	int			*pixelCount,
    int			*error);

void createAveragedImage (
	short int	*inData,
	float		scalingOffset,
    ImageFormat	*format,
	float		*image);

void createFitsImage (
	const char	*regionOrFile,
	const char	*name,
    char	*inFitsName,
    int 	xLength,
    int 	yLength,
    float   *image);

void midiGaussianFit (
	int		counter,
	int		dimension,
	float	*image,
	int		xImage,
	int		yImage,
	int		xP,
	int		yP,
	int		sizeP,
	double	*xT,
	double	*yT,
	double	*sizeXT,
	double	*sizeYT,
    int		*error);

void midiGaussian_1d_fit (
	float	*array,
	int		length,
	int		sizeS,
	float	*centre,
	float	*sizeT,
	float	*fluxErr2Min,
    int		*error);

void midiGetFWHM (
	int		imageCounter,
	float	*image,
	int		xImage,
	int		yImage,
	int		sizeP,
	double	*xT,
	double	*yT,
	double	*sizeXT,
	double	*sizeYT,
    int		*error);

void getBadScansFromSpectrumUndisp (
    FilterData		*filterInfo,
    ImageFormat 	*format,
	CompressedData	*compressed,
	int				*error);

void rejectScansOnWeakSNRUndisp (
    FilterData		*filterInfo,
    ImageFormat 	*format,
	CompressedData	*compressed,
	int				*error);

void removeSkyBackground (
    const char				*shutterId,
    enum ProcessingMode	processing,
    ImageFormat			*format,
    CompressedData		*compressed,
    int					*error);

void getUndispersedPowerSpectrum (
    ImageFormat		*imageFormat,
    CompressedData	*compressed);

long correctTarType (
	const char	*shutterId,
	char 	*tarType,
	float 	*TimeStamp,
	int		length,
	int		*error);

/*********************************************************/
#endif
/*********************************************************/
/*********************************************************/

