#include <cpl.h>

#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "string.h"
#include "midiTableToFits.h"
#include <config.h>
#include "midi_cplupgrade.h"

int table_to_fitsimage(cpl_frameset * frameset,
                              const cpl_parameterlist * parlist,
                              const char * columname,
                              const char * outputfilename,
                              cpl_table * table)
{

   cpl_propertylist    *   qclist;

   int dimenDATA; 
   cpl_array * array_data=NULL;
   cpl_image * image_data_int=NULL;
   cpl_image * image_data=NULL;
   cpl_image * image_stacked=NULL;
   cpl_image * image_stacked_raw=NULL;
   cpl_image * image_stacked_ycollapsed=NULL;
   cpl_imagelist * imagelist_data=NULL;
   cpl_imagelist * imagelist_data_sky=NULL;
   cpl_imagelist * imagelist_data_target=NULL;
   cpl_imagelist * imagelist_data_result=NULL;
   int i;
   int csky=0, ctarget=0;  /* c == counter */
   int maximum_images=0;
   int undefined=0;
   char        *cubename;
   cpl_errorstate          prestate = cpl_errorstate_get();
   char ** target_type; 	
   cpl_type ctype;   

    /* Load extension  Imaging Data  */ 

    dimenDATA=cpl_table_get_column_dimensions(table, columname);
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
 
   if (dimenDATA != 2) {
       return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                         "DATA has a wrong dimension");
    }

    cpl_msg_info(cpl_func, "Number of images in the Table: %" CPL_SIZE_FORMAT "",cpl_table_get_nrow(table));
    cpl_msg_info(cpl_func, "Extracting %" CPL_SIZE_FORMAT " images: please wait ...",cpl_table_get_nrow(table));

    imagelist_data       =cpl_imagelist_new();
    imagelist_data_sky   =cpl_imagelist_new();
    imagelist_data_target=cpl_imagelist_new();
    
/*     Read target type T(arget),S(ky),B(ackground) and store them 
       At the moment TARTYP2 is read as TARTYP1 and TARTYP4 belong to DATA4 and DATA1 
       and therefore are switched  */ 

    if (cpl_table_has_column(table, "TARTYP2"))
    {
       target_type=cpl_table_get_data_string(table, "TARTYP2");
       
    }
    else 
    {
       cpl_msg_warning(cpl_func, "TARTYP2 not found, using TARTYP1");              
       target_type=cpl_table_get_data_string(table, "TARTYP1");
    }
/*     Make sure that the sky and target imagelist have the same length */
/*     so start the comparison only after the first U(ndefined) frame */

    maximum_images=0;
    undefined=0;

    for (i=0; i<cpl_table_get_nrow(table);i++){
      if(strcmp(target_type[i],"U")== 0 ){
          undefined=1;}
        if(strcmp(target_type[i],"S")== 0 && undefined!=0){
          csky++;
       }
       if(strcmp(target_type[i],"T")== 0 && undefined!=0){
          ctarget++;
       }
    }
    
    if (csky>=ctarget) maximum_images=ctarget;
    else maximum_images=csky;
    
    
    csky=0, ctarget=0, undefined=0;
    
    /* Loop over all "images" stored in DATA and add them to the imagelist */

/*     Check the type of the image (float or int) */
    ctype=cpl_table_get_column_type(table, columname);   

    for (i=0; i<cpl_table_get_nrow(table);i++){
       array_data=(cpl_array *)cpl_table_get_array(table,columname, i); 
       
       if(ctype&CPL_TYPE_INT){
          image_data_int=cpl_image_wrap_int(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_int(array_data) );
          /* Cast the image to float */
          image_data=cpl_image_cast(image_data_int, CPL_TYPE_FLOAT);
          cpl_image_unwrap(image_data_int);
       }

       if(ctype&CPL_TYPE_FLOAT){
          image_data_int=cpl_image_wrap_float(cpl_table_get_column_dimension(table,columname,0 ), cpl_table_get_column_dimension(table,columname,1 ), cpl_array_get_data_float(array_data) );
          image_data=cpl_image_cast(image_data_int, CPL_TYPE_FLOAT);
          cpl_image_unwrap(image_data_int);
       }

/*        Append the image to the imagelists */
       cpl_imagelist_set(imagelist_data,image_data,i);

       if(strcmp(target_type[i],"U")== 0 ){
          undefined=1;}
       
       if(strcmp(target_type[i],"S")== 0 && csky<maximum_images && undefined!=0){
          cpl_imagelist_set(imagelist_data_sky,image_data,csky++);
       }
       if(strcmp(target_type[i],"T")== 0 && ctarget<maximum_images && undefined!=0){
          cpl_imagelist_set(imagelist_data_target,image_data,ctarget++);
       }
       
    }
    
    imagelist_data_result=cpl_imagelist_duplicate(imagelist_data_target);
    cpl_imagelist_subtract(imagelist_data_result,imagelist_data_sky); 
    
    /* QC parameters  */
    qclist = cpl_propertylist_new();
/*     qclist = cpl_propertylist_load(cpl_frame_get_filename(rawframe), 0 ); */

    cpl_frame_set_group   (cpl_frameset_get_position(frameset, 0), CPL_FRAME_GROUP_RAW) ;

    cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_CUBE");   
    cubename = cpl_sprintf("%s_%s", columname, outputfilename);
    if(cpl_dfs_save_imagelist(frameset,
                              NULL,
                              parlist,
                              frameset,
                              NULL,
                              imagelist_data,
                              CPL_BPP_IEEE_FLOAT,
                              "midi_raw_to_fitsimage",
                              qclist,
                              NULL,
                              PACKAGE "/" PACKAGE_VERSION,
                              cubename)) {
       /* Propagate the error */
       (void)cpl_error_set_where(cpl_func);
    }


/* Collapse the raw cube by a sigma clipping filter */
    if (cpl_imagelist_get_size(imagelist_data)>1)
    {
       cpl_msg_info(cpl_func, "Saving clipping-collapsed raw fitsfile ...");
       image_stacked_raw=cpl_imagelist_collapse_sigclip_create(imagelist_data, 2.0, 2.0, 0.5, (1<<1), NULL);
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "raw_timecollapsed", outputfilename);



       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_RAWTIMECOLLAPSED");       
       if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL, image_stacked_raw,
                              CPL_BPP_IEEE_FLOAT, "midi_raw_to_fitsimage",
                              qclist, NULL,
                              PACKAGE "/" PACKAGE_VERSION,
                              cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }
    }
    
    cpl_image_delete(image_stacked_raw);


    if(csky>0){
       cpl_msg_info(cpl_func, "Saving sky cube ...");
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "sky", outputfilename);

       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_CUBE_SKY");   
       if(cpl_dfs_save_imagelist(frameset,
                                 NULL,
                                 parlist,
                                 frameset,
                                 NULL,
                                 imagelist_data_sky,
                                 CPL_BPP_IEEE_FLOAT,
                                 "midi_raw_to_fitsimage",
                                 qclist,
                                 NULL,
                                 PACKAGE "/" PACKAGE_VERSION,
                                 cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }
    }

    if(ctarget>0){    
       cpl_msg_info(cpl_func, "Saving target cube ...");
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "target", outputfilename);

       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_CUBE_TARGET");  
       if(cpl_dfs_save_imagelist(frameset,
                                 NULL,
                                 parlist,
                                 frameset,
                                 NULL,
                                 imagelist_data_target,
                                 CPL_BPP_IEEE_FLOAT,
                                 "midi_raw_to_fitsimage",
                                 qclist,
                                 NULL,
                                 PACKAGE "/" PACKAGE_VERSION,
                                 cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }
    }

    if(ctarget>0 && csky>0){
       cpl_msg_info(cpl_func, "Saving (target-sky) cube ...");
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "result", outputfilename);

       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_CUBE_TARGET_MINUS_SKY");   
       if(cpl_dfs_save_imagelist(frameset,
                                 NULL,
                                 parlist,
                                 frameset,
                                 NULL,
                                 imagelist_data_result,
                                 CPL_BPP_IEEE_FLOAT,
                                 "midi_raw_to_fitsimage",
                                 qclist,
                                 NULL,
                                 PACKAGE "/" PACKAGE_VERSION,
                                 cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }
    }
    

    if(ctarget>0 && csky>0){
/* Collapse the result by a sigma clipping filter */
       cpl_msg_info(cpl_func, "Saving clipping-collapsed (target-sky) fitsfile ...");
       image_stacked=cpl_imagelist_collapse_sigclip_create(imagelist_data_result, 2.0, 2.0, 0.5, (1<<1), NULL);
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "result_timecollapsed", outputfilename);

       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_TIMECOLLAPSED");   
       if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL, image_stacked,
                              CPL_BPP_IEEE_FLOAT, "midi_raw_to_fitsimage",
                              qclist, NULL,
                              PACKAGE "/" PACKAGE_VERSION,
                              cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }

    }

    if(ctarget>0 && csky>0){
/* Collapse the result in time by a sigma clipped filter and then in y direction*/
       cpl_msg_info(cpl_func, "Saving clipping-collapsed (target-sky) plus collapsed in y direction ...");
/*        image_stacked_ycollapsed=cpl_image_collapse_window_create(image_stacked,1 ,1 ,260 ,40 ,0); */
       image_stacked_ycollapsed=cpl_image_collapse_create(image_stacked,0);
       cpl_free(cubename);
       cubename = cpl_sprintf("%s_%s_%s", columname, "result_timecollapsed_ycollapsed", outputfilename);

       cpl_propertylist_update_string(qclist, CPL_DFS_PRO_CATG, "MIDI_TIMECOLLAPSED_YCOLLAPSED");   
       if (cpl_dfs_save_image(frameset, NULL, parlist, frameset, NULL, image_stacked_ycollapsed,
                              CPL_BPP_IEEE_FLOAT, "midi_raw_to_fitsimage",
                              qclist, NULL,
                              PACKAGE "/" PACKAGE_VERSION,
                              cubename)) {
          /* Propagate the error */
          (void)cpl_error_set_where(cpl_func);
       }
/*        cpl_image_save(image_stacked_ycollapsed, cubename, CPL_BPP_IEEE_FLOAT, qclist, CPL_IO_DEFAULT); */
       cpl_image_delete(image_stacked_ycollapsed);
       cpl_image_delete(image_stacked);
    }
    
    

/* Free the Memory */

   cpl_propertylist_delete(qclist);
   cpl_free(cubename);

   while(cpl_imagelist_get_size(imagelist_data_sky)>0)
   {
      cpl_imagelist_unset(imagelist_data_sky,0);
   }

   while(cpl_imagelist_get_size(imagelist_data_target)>0)
   {
      cpl_imagelist_unset(imagelist_data_target,0);
   }


   cpl_imagelist_delete(imagelist_data);
   cpl_imagelist_delete(imagelist_data_result);

   cpl_imagelist_delete(imagelist_data_sky);
   cpl_imagelist_delete(imagelist_data_target);


   return (int)cpl_error_get_code();
}
