/**********************************************************
***********************************************************
*				European Southern Observatory
*		     VLTI MIDI Data Reduction Software
*
* Module name:	statistics.h
* Description:  Contains the prototypes for all statistical 
*				computations
*
* History:      
* 22-Dec-03     (csabet) Created
***********************************************************
**********************************************************/
#ifndef _STATISTICS_H
#define _STATISTICS_H

/**********************************************************
*	Include files
**********************************************************/

/**********************************************************
*	Constant definitions
**********************************************************/

/**********************************************************
*	Type definitions 
**********************************************************/

/**********************************************************
*	Global Variables 
**********************************************************/

/**********************************************************
*	Function Prototypes
**********************************************************/
float signalPeak (
	float	*signal,
	int		start,
	int		end,
	int		*peakIndex);

void removeDc (
    int		size,
    float	*inArray,
    float	*outArray);

float signalMean (
	float	*signal,
	int		start,
	int		end);

float signalVariance (
	float	*signal,
	int		start,
	int		end,
	float	*standDev);

float signalMedian (
	float	*signal,
	int		start,
	int		end);

void signalSortInt (
	int	*signal,
	int	start,
	int	end);
	
void signalSort (
	float	*signal,
	int		start, 
	int		end);

void makeStats(
	float	*data,
	int		numdata,
    float	*mean,
    float	*rms);

void midiGetLinearFit (
	double	*x,
	double	*y,
	int		ndata,
    float	sig,
    int		mwt,
    float	*a,
    float	*b,
    float	*siga,
    float	*sigb,
    float	*chi2,
    float	*q,
    int		*error);

float midiGoodnessOfFit (
	float	a,
	float	x,
	int		*error);

void gser (
	float	*gamser,
	float	a,
	float	x,
	float	*gln,
	int		*error);

float gammln (
	float	xx);

void gcf (
	float	*gammcf,
	float	a,
	float	x,
	float	*gln,
	int		*error);
	
/*********************************************************/
#endif
/*********************************************************/
/*********************************************************/

